/*
 * Copyright 2010 Trustees of the University of Pennsylvania Licensed under the
 * Educational Community License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License. You may
 * obtain a copy of the License at
 *
 * http://www.osedu.org/licenses/ECL-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS"
 * BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */
package metridoc.camel.batch

import metridoc.utils.AssertState
import org.apache.camel.Exchange
import org.slf4j.LoggerFactory
import org.codehaus.groovy.runtime.typehandling.GroovyCastException
import javax.sql.DataSource
import metridoc.plugins.sql.SqlPlus

import metridoc.plugins.load.LoadMessage

/**
 * Created by IntelliJ IDEA.
 * User: tbarker
 * Date: 7/26/11
 * Time: 1:31 PM
 */
class BatchIterator implements Iterator<List<Map>> {
    static final VALIDATION_ERROR = "validation_error"
    int batchSize = 50
    Closure<Map> parser
    DataSource dataSource
    /**
     * optional for now.... still deciding if the loading plugin will work without camel
     */
    Exchange originalMessage
    /**
     * optional
     */
    File fileName
    Iterator iterator
    int currentRow
    static final log = LoggerFactory.getLogger(BatchIterator.class)
    boolean notStarted = true
    String groupName

    @Override
    public boolean hasNext() {
        return iterator.hasNext()
    }

    @Override
    public List<Map> next() {

        if (notStarted) {
            validateState()
            notStarted = false
        }

        int currentBatchCount = 0;
        def batch = []
        while (iterator.hasNext() && currentBatchCount < batchSize) {
            currentRow++
            currentBatchCount++
            def next = iterator.next()
            LoadMessage loadMessage = null
            if (parser) {
                try {
                    loadMessage = convertToLoadMessage(next)
                    Map record = parser.call(loadMessage)
                    batch.add(record)

                } catch (GroovyCastException e) {
                    String message = "The return type in the parser given to the BatchIterator is likely wrong, it should be a Map: ${e.message}"
                    log.error(message) //make it a little easier to figure out
                    throw e
                } catch (AssertionError error) {
                    handleValidationError(next, loadMessage, error)
                }
            } else {
                batch.add(next)
            }
        }

        return batch
    }

    private convertToLoadMessage(Object object) {
        return new LoadMessage(originalMessage: originalMessage, body: object, line: currentRow)
    }

    private handleValidationError(record, LoadMessage loadMessage, AssertionError error) {
        log.error("There was a validation error trying to parse ${record}: ${error.getMessage()}")
        def sql
        if (dataSource) {
            log.info("will attempt to store validation error for ${record} into validation_error table")
            try {
                sql = new SqlPlus(dataSource)
                def errorRecord = [
                        file_name: fileName,
                        group_name: groupName,
                        message: error.getMessage(),
                        stack_trace: org.apache.commons.lang.exception.ExceptionUtils.getStackTrace(error),
                        line: loadMessage.line
                ]
                sql.insertBatch("validation_error", errorRecord)
                log.info("successfully stored validation error for ${record} into validation_error table")
            } catch (Exception ex) {
                log.warn("Could not store ${record} into validation_error, the table probably does not exist")
            } finally {
                try {
                    sql.close()
                } catch (Exception ex) {
                    log.warn("Could not close SqlPlus", ex)
                }
            }
        }
    }

    @Override
    public void remove() {
        throw new UnsupportedOperationException("remove not supported in BatchIterator")
    }



    private void validateState() {
        AssertState.notNull(iterator, "The iterator cannot be null in the BatchIterator")

        if (parser) {
            Class[] classes = parser.parameterTypes
            metridoc.utils.AssertState.isTrue(classes.size() == 1, "The parser must have 1 parameter, it has ${classes.size()}, and are ${classes}")
            metridoc.utils.AssertState.isTrue(classes[0].name == "java.lang.Object" || classes[0].name == "metridoc.plugins.load.LoadMessage",
                    "parameter type passed to the parser should be either Object or LoadMessage")
        }
    }
}
