/*
 * Copyright 2010 Trustees of the University of Pennsylvania Licensed under the
 * Educational Community License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License. You may
 * obtain a copy of the License at
 *
 * http://www.osedu.org/licenses/ECL-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS"
 * BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */
package metridoc.plugins.load

import org.slf4j.LoggerFactory
import org.apache.camel.CamelContext
import javax.sql.DataSource
import org.apache.camel.Exchange
import metridoc.utils.Assert
import org.springframework.core.io.Resource
import org.springframework.core.io.ClassPathResource
import metridoc.plugins.schema.SchemaPlugin

import metridoc.plugins.sql.SqlPlus

import metridoc.camel.batch.BatchIterator
import metridoc.camel.batch.ProcessExchangeCallback
import metridoc.camel.batch.ConsumeExchangeHandler

/**
 * Created by IntelliJ IDEA.
 * User: tbarker
 * Date: 7/26/11
 * Time: 1:34 PM
 */
class LoadingPluginRunner implements ProcessExchangeCallback{

    final static log = LoggerFactory.getLogger(LoadingPluginRunner.class)
    CamelContext camelContext
    Binding services
    DataSource dataSource
    Iterator iterator
    String loadingTable
    String normalizationFile
    String validationErrorTable = "error_table"
    String consumeFrom
    int threads = 10
    int batchSize = 50
    Closure parser
    Exchange originalMessage
    String maxMessages = 1
    String fileName
    String schema
    int wait = -1
    String sendTo

    /**
     * determines where this is sent, but also triggers various conventions.  Unless otherwise given, if one were
     * to put in 'foo', it would search and use the following for processing
     *
     * schema: fooSchema.xml
     * iterator: fooIterator
     * normalization: fooNormalization.groovy
     * loading table: foo_loading
     * data source: fooDataSource
     *
     * @param convention specifies the convention that links to various files and services
     * @return the loading plugin runner so you can chain everything
     */
    LoadingPluginRunner sendTo(String convention) {
        sendTo = convention
        return this
    }

    /**
     * provides the parser to use.  With the exception of AssertionErrors normal exceptions from the parser will
     * crash the routine.  Assertion errors will be stored in the error_table as created by the baseSchema.xml
     *
     * @param closure
     * @return
     */
    LoadingPluginRunner eachLine(Closure closure) {
        parser = closure
        return this
    }

    /**
     * runs the routine
     *
     * @throws IllegalArgumentException if sendTo not set
     */
    void run() {

        def handler = new ConsumeExchangeHandler(uri: consumeFrom, camelContext: camelContext, wait: wait, callback: this)
        handler.run()
    }
       //TODO: remove this
//    private void getExchangeFromConsumeFrom() {
//        def context = camelContext
//        def consumer = context.createConsumerTemplate()
//        def consumeFrom = consumeFrom
//        def endpoint = context.getEndpoint(consumeFrom)
//        consumer.getConsumerCache().getConsumer(endpoint) //this will start up the consumer
//
//        def wait = wait
//        switch (wait) {
//            case 0:
//                //get immediately
//                originalMessage = consumer.receiveNoWait(consumeFrom)
//                break;
//            case -1:
//                //wait forever
//                originalMessage = consumer.receive(consumeFrom)
//                break;
//            default:
//                //wait for specified time
//                originalMessage = consumer.receive(consumeFrom, wait)
//        }
//
//    }

    private String getSchema() {
        if (schema) {
            return schema
        }

        schema = "${sendTo}Schema.xml"
    }

    DataSource getDataSource() {

        if (dataSource) {
            return dataSource
        }


        dataSource = services["${sendTo}DataSource"]
    }

    private void runSchema() {
        String schema = getSchema()
        Resource resource = new ClassPathResource(schema)
        boolean schemaExists = resource.exists()
        if (schemaExists) {
            def schemaPlugin = new SchemaPlugin()
            log.debug("loading schema {}", schema)
            def dataSource = getDataSource()
            schemaPlugin.liquibase(dataSource: dataSource, schema: schema)
        } else {
            log.debug("there is no schema to run")
        }
    }

    private void storeData(dataToStore) {
        def sql
        try {
            sql = new SqlPlus(getDataSource())
            sql.insertBatch("${sendTo}_loading", dataToStore)
        } finally {
            try {
                sql.close()
            } catch (Exception e) {
                log.warn("An exception occurred while closing SqlPlus", e)
            }
        }
    }

    void process(Exchange exchange) {
        Assert.notNull(sendTo, "sendTo must be specified for consumeFrom to work")
        log.debug("running load with configuration {}", this)
//        getExchangeFromConsumeFrom()
        originalMessage = exchange
        runSchema()
        if (iterator == null) {
            iterator = metridoc.camel.impl.iterator.IteratorProviderFactory.getIterator(originalMessage)
        }

        Iterator batch = new BatchIterator(batchSize: batchSize, fileName: fileName, originalMessage: originalMessage,
                parser: parser, iterator: iterator, dataSource: dataSource)

        while(batch.hasNext()) {
            def data = batch.next()
            storeData(data)
        }
    }
}
